"""
Contacts app

App for contact analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "contacts"
    tree_root_elementtype = "casualty"
    analysistype =  _('Exposure prevention')
    scenario_displaypermission = 'contacts.display_contactscenarios'
    scenario_editpermission = 'contacts.edit_contactscenarios'
    suggestionlist_displaypermission = 'contacts.display_contactsuggestionlists'
    suggestionlist_editpermission = 'contacts.edit_contactsuggestionlists'
    listurl = 'contacts_scenarios'
    addurl = 'contacts_scenario_add'
    consequence_label = _('Consequence of contact')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': ContactSuggestionlist,
            'scenario': ContactScenario,
            'treeelement': ContactTreeElement,
            'event': ContactEvent,
            'protectionlayer': ContactProtectionLayer,
            'measure': ContactMeasure,
            'protectionlayer_measure': ContactProtectionLayer_Measure,
            'attentionpoint': ContactAttentionPoint,
            'apsuggestionlist': ContactAPSuggestionlist,
        }

class ContactSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_contactsuggestionlists", "May display contact suggestion lists"),
            ("edit_contactsuggestionlists", "May edit contact suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ContactScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(ContactSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_contactscenarios", "May display contact scenarios"),
            ("edit_contactscenarios", "May edit contact scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class ContactTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(ContactScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig


class ContactEvent(base_models.Event):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ContactMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ContactProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(ContactMeasure,
        verbose_name = _('Measures'),
        through="ContactProtectionLayer_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ContactProtectionLayer_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(ContactProtectionLayer)
    measure = versioning_models.VersionedForeignKey(ContactMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ContactAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ContactAttentionPoint(base_models.AttentionPoint):
# todo: remove this field
##    protectionlayer = versioning_models.VersionedForeignKey(ContactProtectionLayer, verbose_name = _('Protection layer'),
##        blank=True, null=True)
    measure = versioning_models.VersionedForeignKey(ContactMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(ContactAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 80)

class Contacts_SuggestionlistSearch(Search):
    permission = 'contacts.display_contactsuggestionlists'
    fields = ['name', 'description']
    model = ContactSuggestionlist
    name = _('Suggestion list')

searches.register(Contacts_SuggestionlistSearch)

class Contacts_ScenarioSearch(Search):
    permission = 'contacts.display_contactscenarios'
    fields = ['name', 'description', 'evaluation']
    model = ContactScenario
    name = _('Scenario')

searches.register(Contacts_ScenarioSearch)

class Contacts_EventSearch(Search):
    permission = 'contacts.display_contactscenarios'
    fields = ['name', 'description']
    model = ContactEvent
    name = _('Cause')

searches.register(Contacts_EventSearch)

class Contacts_ProtectionLayerSearch(Search):
    permission = 'contacts.display_contactscenarios'
    fields = ['name', 'description']
    model = ContactProtectionLayer
    name = _('Protection layer')

searches.register(Contacts_ProtectionLayerSearch)

class Contacts_MeasureSearch(Search):
    permission = 'contacts.display_contactscenarios'
    fields = ['name', 'tag', 'description']
    model = ContactMeasure
    name = _('Measure')

searches.register(Contacts_MeasureSearch)

class Contacts_APSuggestionlistSearch(Search):
    permission = 'contacts.display_contactsuggestionlists'
    fields = ['name', 'description']
    model = ContactAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(Contacts_APSuggestionlistSearch)

class Contacts_AttentionPointSearch(Search):
    permission = 'contacts.display_contactscenarios'
    fields = ['name', 'description']
    model = ContactAttentionPoint
    name = _('Attention point')

searches.register(Contacts_AttentionPointSearch)
