"""
Degradations app

App for degradation analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "degradations"
    tree_root_elementtype = "loc"
    analysistype =  _('Control of degradation')
    scenario_displaypermission = 'degradations.display_degradationscenarios'
    scenario_editpermission = 'degradations.edit_degradationscenarios'
    suggestionlist_displaypermission = 'degradations.display_degradationsuggestionlists'
    suggestionlist_editpermission = 'degradations.edit_degradationsuggestionlists'
    listurl = 'degradations_scenarios'
    addurl = 'degradations_scenario_add'
    consequence_label = _('Consequence of degradation')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': DegradationSuggestionlist,
            'scenario': DegradationScenario,
            'treeelement': DegradationTreeElement,
            'event': DegradationEvent,
            'protectionlayer': DegradationProtectionLayer,
            'measure': DegradationMeasure,
            'protectionlayer_measure': DegradationProtectionLayer_Measure,
            'attentionpoint': DegradationAttentionPoint,
            'apsuggestionlist': DegradationAPSuggestionlist,
        }

class DegradationSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_degradationsuggestionlists", "May display degradation suggestion lists"),
            ("edit_degradationsuggestionlists", "May edit degradation suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class DegradationScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(DegradationSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_degradationscenarios", "May display degradation scenarios"),
            ("edit_degradationscenarios", "May edit degradation scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class DegradationTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(DegradationScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig


class DegradationEvent(base_models.Event):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class DegradationMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class DegradationProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(DegradationMeasure,
        verbose_name = _('Measures'),
        through="DegradationProtectionLayer_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class DegradationProtectionLayer_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(DegradationProtectionLayer)
    measure = versioning_models.VersionedForeignKey(DegradationMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class DegradationAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class DegradationAttentionPoint(base_models.AttentionPoint):
##    protectionlayer = versioning_models.VersionedForeignKey(DegradationProtectionLayer,
##        verbose_name = _('Protection layer'), blank=True, null=True)
    measure = versioning_models.VersionedForeignKey(DegradationMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(DegradationAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 20)

class Degradations_SuggestionlistSearch(Search):
    permission = 'degradations.display_degradationsuggestionlists'
    fields = ['name', 'description']
    model = DegradationSuggestionlist
    name = _('Suggestion list')

searches.register(Degradations_SuggestionlistSearch)

class Degradations_ScenarioSearch(Search):
    permission = 'degradations.display_degradationscenarios'
    fields = ['name', 'description', 'evaluation']
    model = DegradationScenario
    name = _('Scenario')

searches.register(Degradations_ScenarioSearch)

class Degradations_EventSearch(Search):
    permission = 'degradations.display_degradationscenarios'
    fields = ['name', 'description']
    model = DegradationEvent
    name = _('Cause')

searches.register(Degradations_EventSearch)

class Degradations_ProtectionLayerSearch(Search):
    permission = 'degradations.display_degradationscenarios'
    fields = ['name', 'description']
    model = DegradationProtectionLayer
    name = _('Protection layer')

searches.register(Degradations_ProtectionLayerSearch)

class Degradations_MeasureSearch(Search):
    permission = 'degradations.display_degradationscenarios'
    fields = ['name', 'tag', 'description']
    model = DegradationMeasure
    name = _('Measure')

searches.register(Degradations_MeasureSearch)

class Degradations_APSuggestionlistSearch(Search):
    permission = 'degradations.display_degradationsuggestionlists'
    fields = ['name', 'description']
    model = DegradationAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(Degradations_APSuggestionlistSearch)

class Degradations_AttentionPointSearch(Search):
    permission = 'degradations.display_degradationscenarios'
    fields = ['name', 'description']
    model = DegradationAttentionPoint
    name = _('Attention point')

searches.register(Degradations_AttentionPointSearch)
