"""
Fires app

App for fire analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "fires"
    tree_root_elementtype = "damage"
    analysistype =  _('Fire damage limitation')
    scenario_displaypermission = 'fires.display_firescenarios'
    scenario_editpermission = 'fires.edit_firescenarios'
    suggestionlist_displaypermission = 'fires.display_firesuggestionlists'
    suggestionlist_editpermission = 'fires.edit_firesuggestionlists'
    listurl = 'fires_scenarios'
    addurl = 'fires_scenario_add'
    consequence_label = _('Consequence of fire')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': FireSuggestionlist,
            'scenario': FireScenario,
            'treeelement': FireTreeElement,
            'event': FireEvent,
            'protectionlayer': FireProtectionLayer,
            'measure': FireMeasure,
            'protectionlayer_measure': FireProtectionLayer_Measure,
            'attentionpoint': FireAttentionPoint,
            'apsuggestionlist': FireAPSuggestionlist,
        }

class FireSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_firesuggestionlists", "May display fire suggestion lists"),
            ("edit_firesuggestionlists", "May edit fire suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class FireScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(FireSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_firescenarios", "May display fire scenarios"),
            ("edit_firescenarios", "May edit fire scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class FireTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(FireScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig


class FireEvent(base_models.Event):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class FireMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class FireProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(FireMeasure,
        verbose_name = _('Measures'),
        through="FireProtectionLayer_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class FireProtectionLayer_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(FireProtectionLayer)
    measure = versioning_models.VersionedForeignKey(FireMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class FireAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class FireAttentionPoint(base_models.AttentionPoint):
##    protectionlayer = versioning_models.VersionedForeignKey(FireProtectionLayer,
##        verbose_name = _('Protection layer'), blank=True, null=True)
    measure = versioning_models.VersionedForeignKey(FireMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(FireAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 60)

class Fires_SuggestionlistSearch(Search):
    permission = 'fires.display_firesuggestionlists'
    fields = ['name', 'description']
    model = FireSuggestionlist
    name = _('Suggestion list')

searches.register(Fires_SuggestionlistSearch)

class Fires_ScenarioSearch(Search):
    permission = 'fires.display_firescenarios'
    fields = ['name', 'description', 'evaluation']
    model = FireScenario
    name = _('Scenario')

searches.register(Fires_ScenarioSearch)

class Fires_EventSearch(Search):
    permission = 'fires.display_firescenarios'
    fields = ['name', 'description']
    model = FireEvent
    name = _('Cause')

searches.register(Fires_EventSearch)

class Fires_ProtectionLayerSearch(Search):
    permission = 'fires.display_firescenarios'
    fields = ['name', 'description']
    model = FireProtectionLayer
    name = _('Protection layer')

searches.register(Fires_ProtectionLayerSearch)

class Fires_MeasureSearch(Search):
    permission = 'fires.display_firescenarios'
    fields = ['name', 'tag', 'description']
    model = FireMeasure
    name = _('Measure')

searches.register(Fires_MeasureSearch)

class Fires_APSuggestionlistSearch(Search):
    permission = 'fires.display_firesuggestionlists'
    fields = ['name', 'description']
    model = FireAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(Fires_APSuggestionlistSearch)

class Fires_AttentionPointSearch(Search):
    permission = 'fires.display_firescenarios'
    fields = ['name', 'description']
    model = FireAttentionPoint
    name = _('Attention point')

searches.register(Fires_AttentionPointSearch)
