# substance views

from analyses.main import get_scenarios_for_scope, build_scenariotree
from analyses.main import analysesmodules

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.datastructures import SortedDict
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.utils.text import capfirst

from installations import models as app_models
from installations import forms as app_forms
from installations import tables as app_tables
from installations.views.main import installations_navigationlist, get_visible_objects

from utils import crud
from utils import forms as utils_forms
from utils.permissions import redirect_to_not_allowed
from utils.trees import views as trees_views
from utils.versioning.helpers import annotate_versioned
from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion

class MixIn(object):
    "Common items for object views"
    model = app_models.Scope

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['breadcrumbs'] = object.get_breadcrumbs()

    tabs = [
        {'name': 'scope',
            'label': _('Scope')},
        {'name': 'sections',
            'label': _('Sections')},
        {'name': 'scenarios',
            'label': _('Scenarios')},
    ]

class display_base(MixIn, versionedcrud.DisplayObjectView):
    allow_publish = False
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        if self.allow_publish and object.may_publish(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_publish_url(), 'label': _('Publish')})

class display_scope(display_base):
    form_class = app_forms.ScopeSheet
    allow_publish = True

class display_sections(display_base):
    allow_delete = False
    form_class = None
    template = 'installations/scope_sections.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_sections, self).add_to_extra_context(extra_context, object)

        queryset = app_models.SectionTreeElement.tree.all().get_versionedobjects(self.version)
        sections= object.sections.all().values_list('v_id', flat=True)
        for item in queryset:
            item.selected = False
            if item.section.v_id in sections:
                item.selected = True
        res = get_visible_objects(queryset)
        annotate_versioned(res, 'section')
        extra_context['objects'] = res

class display_scenarios(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    template = 'installations/section_scenarios.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_scenarios, self).add_to_extra_context(extra_context, object)
        analysistype = self.request.session.get('analysistype')
        extra_context['scenarios'] = build_scenariotree(get_scenarios_for_scope(object, self.request.user, self.version), analysistype=analysistype)
        if analysesmodules.show_analysis_add_choices(self.request.user):
            extra_context['buttons'] = [{
                    'url': object.get_add_scenario_url(),
                    'label': _('Add'),
                }]


class display(MixIn, crud.TabContainer):
    tabviews = {
        'scope': display_scope(),
        'sections': display_sections(),
        'scenarios': display_scenarios(),
    }
    default = 'scope'

class edit_scope(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.ScopeForm
    tabs = []

class edit_sections(MixIn, trees_views.edit_tree):
    treeelement_class = app_models.SectionTreeElement
    template = 'installations/scope_sections_edit.html'
    helptext = _('Click on a section to add it to or remove it from this scope.')

    def get_versioned_treeobjects(self, object):
        queryset = app_models.SectionTreeElement.get_authorized_tree(self.request.user).get_versionedobjects(self.version)
        sections= object.sections.all().values_list('v_id', flat=True)
        for item in queryset:
            item.selected = False
            if item.section.v_id in sections:
                item.selected = True
        res = get_visible_objects(queryset)
        annotate_versioned(res, 'section')
        return res

    def add_to_extra_context(self, extra_context, object):
        trees_views.edit_tree.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

    def setsection(self, item):
        success = False
        item = int(item)
        scope = self.object
        if item:
            try:
                section = app_models.Section.objects.get_versionedobject(v_id=item, version=WorkVersion)
            except app_models.Section.DoesNotExist:
                section = None
            if section:
                ss = [item.section_id for item in app_models.Scope_Section.objects.filter(scope=scope).get_versionedobjects(version=WorkVersion)]
                member = section.v_id in ss
                if not member:
                    try:
                        # watch out: this needs to return also the deleted objects:
                        ss = app_models.Scope_Section.objects.get(scope=scope, section=section, version__isnull=True)
                        ss.deleted = False
                        ss.save(self.request.user)
                    except app_models.Scope_Section.DoesNotExist:
                        ss = app_models.Scope_Section(scope=scope, section=section)
                        ss.save(self.request.user)
                else:
                    ss = app_models.Scope_Section.objects.get_versionedobject(scope=scope, section=section, version=WorkVersion)
                    ss.delete(self.request.user)
                success = True
        return success


class edit(MixIn, crud.TabContainer):
    tabviews = {
        'scope': edit_scope(),
        'sections': edit_sections(),
    }
    default = 'scope'

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.ScopeSheet
    tabs = []

    def get_default_url(self, object):
        return reverse('installations_scopes')

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.ReportObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class add(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.ScopeForm
    tabs = []

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url('sections'))

    def add_to_extra_context(self, extra_context, object):
        # overridden: do not add breadcrumbs here
        extra_context['navigationlist'] =  installations_navigationlist()

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.ScopeSheet

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class ScopeFilter(crud.Filter):
    form = app_forms.ScopeFilterForm
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
            ]

class ScopeListObject(versionedcrud.TableListObject):
    defaultsort = 'name'
    table_class = app_tables.ScopesTable
    queryset = app_models.Scope.objects.filter(auto=False)
    title = _('Scopes')

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('scopes'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('scopes'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class scopes(versionedcrud.TableListView):
    "List of scopes"

    table_class = app_tables.ScopesTable
    defaultsort = 'name'
    queryset = app_models.Scope.objects.filter(auto=False)
    paginate_by = 100
    title = _('Scopes')
    filter = ScopeFilter()
    redirect_on_unique = True
    allow_pdf = True
    pdfsendmail = True
    pdfselectform_class = utils_forms.ListReportForm
    listobject = ScopeListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Scope)

    def add_to_extra_context(self, extra_context):
        super(scopes, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  installations_navigationlist()
        if (self.version==WorkVersion) and app_models.Scope.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('installations_scope_add'),
                    'label': _('Add'),
                }]


class add_scenario(versionedcrud.EditObjectView):
    model = app_models.Scope
    objectpermission = None
    form_class = app_forms.AddScenarioForm

    def get_form(self, object, **formargs):
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES, scope=object, version=WorkVersion, user=self.request.user)
        else:
            form = self.form_class(initial=self.get_initial(object), scope=object, version=WorkVersion, user=self.request.user)
        return form


    def get_initial(self, object):
        initial = {}
        at = self.request.session.get('analysistype')
        if at:
            initial['analysistype'] = at
        return initial

    def save_form(self, form):
        self.analysistype = form.cleaned_data['analysistype']
        self.use_suggestionlist = form.cleaned_data['use_suggestionlist']
        return self.object

    def set_message(self, object):
        pass

    def get_redirect(self, refform, object):
        if self.use_suggestionlist:
            return HttpResponseRedirect(analysesmodules[self.analysistype].get_model('scenario').get_add_to_scope_via_suggestionlist_url(scope=object))
        return HttpResponseRedirect(analysesmodules[self.analysistype].get_model('scenario').get_add_to_scope_url(scope=object))

    def add_to_extra_context(self, extra_context, object):
        pass

class publish(MixIn, versionedcrud.PublishObjectView):
    tabs = []
    vfkfields_to_check = []
    # TODO: add a check on the related sections here ?
