# installation views

from analyses.main import get_scenarios_for_scopes, build_scenariotree
from analyses.main import analysesmodules

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.utils.text import capfirst

from hazards import models as hazards_models
from hazards import tables as hazards_tables

from installations import models as app_models
from installations import forms as app_forms
from installations import tables as app_tables
from installations.views.main import installations_navigationlist

from utils import crud
from utils import forms as utils_forms
from utils import permissions
from utils.trees import views as trees_views
from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion

class MixIn(object):
    "Common items for object views"
    model = app_models.Section
    include_attachments = True
    include_links = True

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  installations_navigationlist()
        breadcrumbs = [{'url': item.get_absolute_url(), 'label': u'%s' % item} for item in object.get_ancestors()]
        breadcrumbs.append({'label': u'%s' % object})
        extra_context['breadcrumbs'] = breadcrumbs

    basetabs = [
        {'name': 'section',
            'label': _('Section')},
        {'name': 'weakpoints',
            'label': _('Weak points')},
        {'name': 'subsections',
            'label': _('Subsections')},
        {'name': 'substances',
            'label': _('Substances')},
        {'name': 'reactions',
            'label': _('Reactions')},
        {'name': 'scenarios',
            'label': _('Scenarios')},
    ]

    def setup_tabs(self):
        self.tabs = self.basetabs[:]
        if self.object and self.object.may_change(self.request.user):
            if self.object.is_root_node():
                self.tabs.append(
                    {'name': 'permissions',
                    'label': _('Permissions')},
                )

class display_base(MixIn, versionedcrud.DisplayObjectView):
    allow_publish = False
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        if self.allow_publish and object.may_publish(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_publish_url(), 'label': _('Publish')})

class display_section(display_base):
    form_class = app_forms.SectionSheet
    allow_publish = True

class display_weakpoints(display_base):
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        super(display_weakpoints, self).add_to_extra_context(extra_context, object)
        queryset = object.weakpoint_set.all()
        table = app_tables.WeakpointsTable(queryset)
        extra_context['table'] = table

class display_subsections(display_base):
    allow_change = True
    allow_delete = False
    form_class = None
    template = 'installations/section_subsections.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_subsections, self).add_to_extra_context(extra_context, object)
        el = object.get_treeelement()
        queryset = el.get_descendants(include_self=True).get_versionedobjects(version=el._version).select_related_versioned()
        extra_context['objects'] = queryset

class display_substances(display_base):
    allow_change = False
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        super(display_substances, self).add_to_extra_context(extra_context, object)
        # aggregate all substances from lower sections
        queryset = object.get_section_substances()
        table = app_tables.Section_SubstancesTable(queryset)
        if object.may_change(self.request.user):
            table.buttons = [{
                    'url': object.get_add_substance_url(),
                    'label': _('Add'),
                }]
        extra_context['table'] = table

class display_reactions(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    template = 'installations/section_reactions.html'

    def comp(self, s1, s2):
        if s1.universal and not s2.universal:
            return 1
        if s2.universal and not s1.universal:
            return -1
        if s1.name < s2.name:
            return -1
        if s1.name == s2.name:
            return 0
        return 1

    def add_to_extra_context(self, extra_context, object):
        super(display_reactions, self).add_to_extra_context(extra_context, object)
        # aggregate all reactions from lower sections
        table = app_tables.Section_ReactionsTable(object.get_section_reactions())
        if object.may_change(self.request.user):
            table.buttons = [{
                    'url': object.get_add_reaction_url(),
                    'label': _('Add'),
                }]
        extra_context['table'] = table

        substances, reactions = object.get_substances_and_reactions()
        reaction_dict = {}
        for int in reactions:
            if self.comp(int.substance1, int.substance2) < 0:
                reaction_dict.setdefault((int.substance1.id, int.substance2.id),[]).append(int)
            else:
                reaction_dict.setdefault((int.substance2.id, int.substance1.id),[]).append(int)

        rows = []
        count = substances.count()
        subst = substances.order_by('-universal', '-name')
        i = 0
        for item in substances.order_by('universal', 'name'):
            x = []
            j = 0
            for sub in subst:
                r = reaction_dict.get((item.id, sub.id), [])
                a = u''
                if r:
                    a = u'used'
                elif j <= (count - i - 1):
                    a = u'available'
                x.append({'reactions': r, 'class': a})
                j += 1
            rows.append({'substance': item, 'cell': x})
            i += 1
        matrix = {'substances': subst, 'rows': rows}
        extra_context['matrix'] = matrix


class display_scenarios(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    template = 'installations/section_scenarios.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_scenarios, self).add_to_extra_context(extra_context, object)
        scopes = list(object.get_scopes())
        asco = object.get_auto_scope()
        analysistype = self.request.session.get('analysistype')
        extra_context['scenarios'] = build_scenariotree(get_scenarios_for_scopes(scopes, self.request.user, self.version), scope=asco, analysistype=analysistype)
        if analysesmodules.show_analysis_add_choices(self.request.user):
            extra_context['buttons'] = [{
                    'url': object.get_add_scenario_url(),
                    'label': _('Add'),
                }]


class display_permissions(display_base):
    allow_change = True
    allow_delete = False
    form_class = app_forms.SectionPermissionsSheet

    def check_permission(self, object):
        # only for root nodes
        if not object.is_root_node():
            return permissions.redirect_to_not_allowed()
        if not object.may_change(self.request.user):
            return permissions.redirect_to_not_allowed()
        return super(display_permissions, self).check_permission(object)

class display(MixIn, crud.TabContainer):
    tabviews = {
        'section': display_section(),
        'weakpoints': display_weakpoints(),
        'subsections': display_subsections(),
        'substances': display_substances(),
        'reactions': display_reactions(),
        'scenarios': display_scenarios(),
        'permissions': display_permissions(),
    }
    default = 'section'

class edit_section(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.SectionForm
    basetabs = []

class WeakpointsInline(versionedcrud.Inline):
    formset_class = app_forms.InlineWeakpointsFormSet

class edit_weakpoints(MixIn, versionedcrud.EditObjectView):
    form_class = None
    basetabs = []
    inlines = [
            WeakpointsInline(),
        ]


class edit_subsections(MixIn, trees_views.edit_tree):
    treeelement_class = app_models.SectionTreeElement
    template = 'installations/section_subsections_edit.html'
    helptext = _('Drag and drop the sections to rearrange the tree. Right-click to open the context menu.')

    def add_to_extra_context(self, extra_context, object):
        trees_views.edit_tree.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

    def create_element(self, parent, type, name, treeelementkwargs={}):
        if type == 'section':
            instance = app_models.Section(name=name)
            return instance.save(self.request.user, parent=parent)

    def rename_element(self, element, name):
        element.section.name = name
        element.section.save(self.request.user)

    def get_treeobjects(self, object):
        el = object.sectiontreeelement_set.all()[0]
        return el.get_descendants(include_self=True).get_versionedobjects(version=el._version).select_related_versioned()

class edit_permissions(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.SectionPermissionsForm
    basetabs = []
    template = 'installations/section_permissions_edit.html'

    def check_permission(self, object):
        # only for root nodes
        if not object.is_root_node():
            return permissions.redirect_to_not_allowed()
        return super(edit_permissions, self).check_permission(object)

    def save_form(self, form):
        object = form.save(commit=False)
        # omit the real form save: this tab page only changes the m2m
        # this avoids setting the object's dirty flag
        form.save_m2m()
        return object

class edit(MixIn, crud.TabContainer):
    tabviews = {
        'section': edit_section(),
        'weakpoints': edit_weakpoints(),
        'subsections': edit_subsections(),
        'permissions': edit_permissions(),
    }

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.SectionSheet
    def setup_tabs(self):
        self.tabs = []
    def get_default_url(self, object):
        return reverse('installations_overview')

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.ReportObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class add(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.SectionForm
    basetabs = []

    def setup_tabs(self):
        self.tabs = self.basetabs[:]

    def add_to_extra_context(self, extra_context, object):
        # overridden: do not add breadcrumbs here
        extra_context['navigationlist'] =  installations_navigationlist()

class add_to_section(add):
    parent_class = app_models.Section
    parent_field = 'parent'
    parent_tab = 'subsections'

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.SectionSheet
    basetabs = []

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class add_scenario(versionedcrud.EditObjectView):
    model = app_models.Section
    objectpermission = None
    form_class = app_forms.AddScenarioForm

    def get_form(self, object, **formargs):
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES, section=object, version=WorkVersion, user=self.request.user)
        else:
            form = self.form_class(initial=self.get_initial(object), section=object, version=WorkVersion, user=self.request.user)
        return form

    def get_initial(self, object):
        initial = {'scope': object.get_auto_scope().v_id}
        at = self.request.session.get('analysistype')
        if at:
            initial['analysistype'] = at
        return initial

    def save_form(self, form):
        self.analysistype = form.cleaned_data['analysistype']
        self.scope = form.cleaned_data['scope']
        self.use_suggestionlist = form.cleaned_data['use_suggestionlist']
        return None

    def set_message(self, object):
        pass

    def get_redirect(self, refform, object):
        if self.use_suggestionlist:
            return HttpResponseRedirect(analysesmodules[self.analysistype].get_model('scenario').get_add_to_scope_via_suggestionlist_url(scope=self.scope))
        return HttpResponseRedirect(analysesmodules[self.analysistype].get_model('scenario').get_add_to_scope_url(scope=self.scope))

    def add_to_extra_context(self, extra_context, object):
        pass

class publish(MixIn, versionedcrud.PublishObjectView):
    def setup_tabs(self):
        self.tabs = []

# add form to select subsections too or not