"""
forms for users app
"""

from django import forms
from django.contrib.auth.models import Permission
import django.contrib.admin.widgets as adminwidgets
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from users import models as app_models

from utils import forms as utils_forms
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils import widgets as utils_widgets
from utils import sheets

class PermissionManyToManyField(sheets.ManyToManyField):
    def format_value(self, value):
        return u'%s' % value.name

class GroupManyToManyField(sheets.ManyToManyField):
    def get_link(self, object):
        return reverse('users_group', args=[object.id,'group'])

class UserFilterForm(utils_lists.FilterForm):
    """Custom filterform for the users list"""
    username = forms.CharField(max_length=100, required=False, label=_('Username'))

class UserForm(utils_forms.ModelForm):
    username = forms.RegexField(label=_("Username"), max_length=30, regex=r'^[\w.@+-]+$',
        help_text = _("Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only."),
        error_messages = {'invalid': _("This value may contain only letters, numbers and @/./+/-/_ characters.")})
    groups = forms.ModelMultipleChoiceField(app_models.Group.objects.all(),
        label=_('Groups'),
        required=False,
        widget = adminwidgets.FilteredSelectMultiple(verbose_name=_('groups'),
            is_stacked=False)
    )
    user_permissions = forms.MultipleChoiceField(choices = [
        (i.id, i.name) for i in Permission.objects.exclude(
            codename__startswith="add").exclude(codename__startswith="change"
            ).exclude(codename__startswith="delete")],
        label=_('Permissions'),
        required=False,
        widget = adminwidgets.FilteredSelectMultiple(verbose_name=_('permissions'),
            is_stacked=False)
    )
    class Meta:
        model = app_models.User
        fields = ('username', 'last_name', 'first_name', 'email', 'groups',
            'user_permissions', 'is_active', 'is_superuser')

class UserSheet(sheets.Sheet):
    groups = GroupManyToManyField(data='groups', name='groups', label=_('Groups'))
    user_permissions = PermissionManyToManyField(data='user_permissions',
        name='user_permissions', label=_('Permissions'),
        newlines=True)
    class Meta:
        model = app_models.User
        fields = ('username', 'last_name', 'first_name', 'email', 'groups',
            'user_permissions', 'is_active', 'is_superuser')

class GroupFilterForm(utils_lists.FilterForm):
    """Custom filterform for the users list"""
    name = forms.CharField(max_length=100, required=False, label=_('Name'))

class GroupForm(utils_forms.ModelForm):
    permissions = forms.MultipleChoiceField(choices = [
        (i.id, i.name) for i in Permission.objects.exclude(
            codename__startswith="add").exclude(codename__startswith="change"
            ).exclude(codename__startswith="delete")],
        label=_('Permissions'),
        required=False,
        widget = adminwidgets.FilteredSelectMultiple(verbose_name=_('permissions'),
            is_stacked=False)
    )
    class Meta:
        model = app_models.Group
        fields = ('name', 'permissions',)
        attrs = {
            'name': {'size':75},
        }

class GroupSheet(sheets.Sheet):
    permissions = PermissionManyToManyField(data='permissions', name='permissions',
        label=_('Permissions'), newlines=True)
    class Meta:
        model = app_models.Group
        fields = ('name', 'permissions',)


