"""
Tests for links app
"""

import datetime

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType

import time

import utils.links.links_test.models as test_models
from utils.links.models import Link
import utils.links.views as test_views
from utils import tests as utils_tests
from utils.versioning.models import Version, WorkVersion
from utils.versioning.versioning_test import models as vt_models

class LinksTestCase(test.TestCase):
    fixtures = ['dummyusers',]

    def setUp(self):
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        s = test_models.LinkStreet(name='First')
        s.save(User.objects.get(pk=1))

class test_links(LinksTestCase):

    def test_views(self):
        csid = test_models.LinkStreet.objects.all()[0].id
        csvid = test_models.LinkStreet.objects.all()[0].v_id

        self.assertEquals(test_views.add_url_for_obj(test_models.LinkStreet.objects.get(pk=csid)), "/links/add-for/links_test/linkstreet/%s/" % csvid)
        self.assertEquals(test_views.display_url_for_obj(test_models.LinkStreet.objects.get(pk=csid)), "/links/display-for/links_test/linkstreet/%s/" % csvid)

    def test_link_views(self):
        base_url = '/links/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2)

        csid = test_models.LinkStreet.objects.all()[0].id
        csvid = test_models.LinkStreet.objects.all()[0].v_id

        class link_add(utils_tests.add_TestDefinition):
            description = 'add an link'
            url = base_url + 'add-for/links_test/linkstreet/%s/' % csvid
            users = [1,]
            data = {
                "name": 'planop',
                "url": 'http://www.planop.be',
                "management-referer": base_url + 'display-for/links_test/linkstreet/%s/' % csvid
            }
            redirect_url = base_url + 'display-for/links_test/linkstreet/%s/' % csvid
            def posttest(self, testcase, allowed):
                if allowed:
                    Link.objects.order_by('-id').get_versionedobjects(version=WorkVersion)[0].delete(testcase.user, override=True)

        class link_delete(utils_tests.delete_TestDefinition):
            description = 'delete an link'
            users = [1,]
            redirect_url = base_url + 'display-for/links_test/linkstreet/%s/' % csvid
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = Link(name='planop', url='http://www.planop.be', content_object=test_models.LinkStreet.objects.get(pk=csid))
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    Link.objects.get_versionedobject(id=self.pk, version=WorkVersion).delete(defaultuser, override=True)

        class links_display(utils_tests.TestDefinition):
            description = 'display links for street 1'
            url = base_url + 'display-for/links_test/linkstreet/%s/' % csvid
            users = [1,2,]
            contains = [
                {'url': base_url + 'add-for/links_test/linkstreet/%s/' % csvid, 'users': [1,]},
            ]


        tests = [
            link_add(),
            link_delete(),
            links_display(),
        ]

        utils_tests.run_tests(self, users, tests)


    def test_templatetags(self):
        from django.template import Context, Template
        csid = test_models.LinkStreet.objects.all()[0].id

        d = {'MEDIA_URL': 'x', 'object': test_models.LinkStreet.objects.get(pk=csid)}

        t = Template("""
            {% load links %}
            {% links_link object %}
        """)
        response = t.render(Context(d))

class LinksPublishTestCase(LinksTestCase):
    def test_publish(self):
        """
        Test that when publishing a single object, the links for that object are also published
        """
        user = User.objects.get(pk=1)

        csid = test_models.LinkStreet.objects.all()[0].id
        csvid = test_models.LinkStreet.objects.all()[0].v_id

        self.assertEquals(vt_models.Building.objects.count(), 0)
        self.assertEquals(vt_models.Building.objects.next_v_id(), 1)

        b1 = vt_models.Building(name='b1')
        b1.save(user)

        b2 = vt_models.Building(name='b2')
        b2.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 2)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 0)

        v1 = Version(name='v1', published=True)
        v1.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 4)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 2)

        # add link to b1
        b1 = vt_models.Building.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        a = Link(content_object=b1, url='http://www.google.be')
        a.save(user)

        # add link to b2
        b2 = vt_models.Building.objects.get_versionedobject(v_id = 2, version=WorkVersion)
        a = Link(content_object=b2, url='http://www.google.be')
        a.save(user)

        self.assertEquals(Link.objects.count(), 2)
        self.assertEquals(Link.objects.dirty().count(), 2)
        self.assertEquals(Link.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(Link.objects.filter(published=True).count(), 0)

        # create a version of b1
        time.sleep(0.01)
        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        b2.publish(user, v2)

        self.assertEquals(Link.objects.count(), 3)
        self.assertEquals(Link.objects.dirty().count(), 1)
        self.assertEquals(Link.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(Link.objects.filter(published=True).count(), 1)
