# generic tree views

from django import forms
from django.forms.forms import NON_FIELD_ERRORS
from django.utils.hashcompat import md5_constructor
from django.utils.translation import ugettext_lazy as _

from mptt import utils as mptt_utils

from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion
from utils.trees import models as trees_models

try:
    import cPickle as pickle
except ImportError:
    import pickle


class EditTreeForm(forms.Form):
    """Custom form to keep causetree actions"""
    actions = forms.CharField(required=False, widget=forms.Textarea)
    hash = forms.CharField(max_length=100, required=True)
    def __init__(self, hash=None, *args, **kwargs):
        super(EditTreeForm, self).__init__(*args, **kwargs)
        self.hash = hash
    def clean(self):
        if self.cleaned_data.get("hash") != self.hash:
            del self.data['actions']
            raise forms.ValidationError(_('Unrecoverable error: the tree was changed since you started editing it.'))
        return self.cleaned_data

class edit_tree(versionedcrud.EditObjectView):
    form_class = EditTreeForm
    treeelement_class = None
    tempids = {}
    currenttempid = -1
    helptext = _('Drag and drop the items to rearrange the tree. Right-click to open the context menu.')

# Actions

    def move(self, item, refitem=None, location='inside'):
        success = False
        item = int(item)
        if refitem:
            refitem = int(refitem)
        if item < 0:
            item = self.tempids.get(item)
        if refitem and refitem < 0:
            refitem = self.tempids.get(refitem)
        if self.treeelement_class and item:
            try:
                treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
            except self.treeelement_class.DoesNotExist:
                treeelement = None
            if treeelement:
                if refitem:
                    try:
                        target = self.treeelement_class.objects.get_versionedobject(v_id=refitem, version=WorkVersion)
                    except self.treeelement_class.DoesNotExist:
                        target = None
                else:
                    target = None
            if treeelement:
                position = {'before': 'left', 'after': 'right', 'inside': 'last-child'}.get(location,'last-child')
                try:
                    treeelement.move_to(self.request.user, target, position)
                    success = True
                except:
                    success = False
        return success

    def create_element(self, parent, type, name, treeelementkwargs={}):
        """
        Override this to add different element types
        """
        pass
##        newitem = self.treeelement_class(parent=parent,
##                    elementtype = xxx, **treeelementkwargs)
##        newitem.save(self.request.user)
##        return newitem

    def add(self, item, type=None, name=u''):
        success = False
        if item:
            item = int(item)
        if item < 0:
            item = self.tempids.get(item)
        if self.treeelement_class and type:
            if item:
                try:
                    treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
                except self.treeelement_class.DoesNotExist:
                    treeelement = None
            else:
                treeelement=None
            newelement = self.create_element(parent=treeelement, type=type, name=name[:100])
            if newelement:
                self.tempids[self.currenttempid] = newelement.v_id
                self.currenttempid -= 1
                success = True
        return success

    def insert(self, item, type=None, name=u''):
        success = False
        if item:
            item = int(item)
        if item < 0:
            item = self.tempids.get(item)
        if self.treeelement_class and type:
            if item:
                try:
                    treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
                except self.treeelement_class.DoesNotExist:
                    treeelement = None
            else:
                treeelement=None
            if treeelement:
                newelement = self.create_element(parent=treeelement.parent, type=type, name=name[:100])
                if newelement:
                    newelement.move_to(self.request.user, treeelement, 'right')
                    self.tempids[self.currenttempid] = newelement.v_id
                    self.currenttempid -= 1
                    # reload
                    treeelement = self.treeelement_class.objects.get(id=treeelement.id)
                    newelement = self.treeelement_class.objects.get(id=newelement.id)
                    treeelement.move_to(self.request.user, newelement, 'last-child')
                    success = True
        return success

    def togglegate(self, item):
        success = False
        item = int(item)
        if item < 0:
            item = self.tempids.get(item)
        if self.treeelement_class:
            try:
                treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
            except self.treeelement_class.DoesNotExist:
                treeelement = None
            if treeelement:
                treeelement.gate = not treeelement.gate
                treeelement.save(self.request.user)
                success = True
        return success

    def delete(self, item):
        success = False
        item = int(item)
        if item < 0:
            item = self.tempids.get(item)
        if self.treeelement_class:
            try:
                treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
            except self.treeelement_class.DoesNotExist:
                treeelement = None
            if treeelement:
                treeelement.delete(self.request.user)
                success = True
        return success

    def rename_element(self, element, name):
        """
        Override this to rename the linked object
        """
        element.content_object.name = name[:100]
        element.content_object.save(self.request.user)

    def rename(self, item, name):
        success = False
        item = int(item)
        if item < 0:
            item = self.tempids.get(item)
        if self.treeelement_class:
            try:
                treeelement = self.treeelement_class.objects.get_versionedobject(v_id=item, version=WorkVersion)
            except self.treeelement_class.DoesNotExist:
                treeelement = None
            if treeelement:
                self.rename_element(treeelement, name)
                success = True
        return success

    # crud overrides
    def get_form(self, object, **formargs):
        # do not pass in instance, since we are not using a modelform
        self.treeobjects = self.get_versioned_treeobjects(object)
        hash = security_hash(self.treeobjects)
        if self.request.method == 'POST':
            # secret: make copy of POST dict so we can change it in case of errors
            form = self.form_class(data=self.request.POST.copy(), files=self.request.FILES, hash=hash)
        else:
            form = self.form_class(initial={'hash': hash})
        self.setup_form(form, object)
        return form

    def save_form(self, form):
        actions = form.cleaned_data['actions'].split(u';')
        for action in actions:
            if action:
                ac = action.split("|")
                # unescape
                ac = map(unescape, ac)
                getattr(self, ac[0])(*ac[1:])
        return self.object

    def add_to_extra_context(self, extra_context, object):
        super(edit_tree, self).add_to_extra_context(extra_context, object)
        extra_context['treeobjects'] = self.treeobjects
        extra_context['helptext'] = self.helptext

    def get_versioned_treeobjects(self, object):
        return list(self.get_treeobjects(object).get_versionedobjects(version=self.version).select_related_versioned())

    def get_treeobjects(self, object):
        """
        return the treeobjects
        does not require calling get_versioned
        """
        return self.treeelement_class.objects.none()

def security_hash(data):
    pickled = pickle.dumps(data, pickle.HIGHEST_PROTOCOL)
    return md5_constructor(pickled).hexdigest()

def unescape(data):
    return data.replace(u'%7C', u'|').replace(u'%3B', u';')

def annotate_tree(treeobjects):
    """
    0: space, 1: line, 2: cross, 3: corner

    """
    res = []
    rghts = {}
    images = ""
    for previous, current, next in mptt_utils.previous_current_next(addgates(treeobjects)):

        if previous:
            closed_levels = range(previous['level'], current['level'], -1)
        else:
            closed_levels = range(current['level'], 0, -1)

        if next:
            new_level = current['level'] < next['level']
        else:
            new_level = False

        rghts[current['level']] = current['rght']
        if current['level'] > 0:
            parent_rght = rghts[current['level'] - 1]
            siblings = parent_rght > (current['rght'] + 1)
        else:
            siblings = False

        for item in closed_levels:
            images = images[:-1]

        if siblings:
            current['treeimages'] = images + '2'
        else:
            current['treeimages'] = images + '3'
        if new_level:
            if siblings:
                images = images + '1'
            else:
                images = images + '0'
        res.append(current)
    return res

def addgates(treeobjects):

    res = []
    gates = []
    for previous, current, next in mptt_utils.previous_current_next(treeobjects):
        newitem = {'item': current}
        newitem['get_elementtype_for_tree'] = current.get_elementtype_for_tree()
        newitem['tree_display'] = current.tree_display()
        newitem['rght'] = current.rght

        extra = 0
        for g in gates:
            if current.rght < g:
                extra += 1
        newitem['level'] = current.level + extra
        res.append(newitem)

        if next and next.level == current.level + 1:
            if current.rght <> next.rght + 1:
                if current.gate:
                    newitem = {'get_elementtype_for_tree': 'andgate', 'tree_display': _('AND')}
                else:
                    newitem = {'get_elementtype_for_tree': 'orgate', 'tree_display': _('OR')}
                newitem['rght'] = current.rght
                newitem['level'] = current.level + extra + 1
                res.append(newitem)
                gates.append(current.rght)
    return res