"""
utility decorator functions
"""
from django.core.urlresolvers import reverse
from django.core.urlresolvers import RegexURLPattern, RegexURLResolver
from django.conf.urls.defaults import patterns
import inspect
import logging

class DecoratedURLPattern(RegexURLPattern):
    """Wrapper for a RegexURLPattern with decorator
    used by decorated_patterns function
    """
    def resolve(self, *args, **kwargs):
        result = RegexURLPattern.resolve(self, *args, **kwargs)
        if result:
            result = list(result)
            if inspect.isfunction(result[0]):
                result[0] = self._decorate_with(result[0])
            else:
                result[0] = self._decorate_with(result[0].__call__)
        return result

class DecoratedURLResolver(RegexURLResolver):
    """Wrapper for a RegexURLResolver with decorator
    used by decorated_patterns function
    """
    def resolve(self, *args, **kwargs):
        result = RegexURLResolver.resolve(self, *args, **kwargs)
        if result:
            result = list(result)
            if inspect.isfunction(result[0]):
                result[0] = self._decorate_with(result[0])
            else:
                result[0] = self._decorate_with(result[0].__call__)
        return result

def decorated_patterns(prefix, func, *args):
    """
    utility to decorate an entire list of patterns, most likely with the
    permission_required decorator

    example usage in urls.py:

    urlpatterns = decorated_patterns('', permission_required('hazards.display_hazards'),
        url(...

    """
    result = patterns(prefix, *args)
    if func:
        for p in result:
            if isinstance(p, RegexURLPattern):
                p.__class__ = DecoratedURLPattern
                p._decorate_with = func
            if isinstance(p, RegexURLResolver):
                p.__class__ = DecoratedURLResolver
                p._decorate_with = func
    return result



def absolute_url(object, tab=None):
    """
    Helper function to get the absolute url from an object.
    Calls get_absolute_url method of the object
    Returns empty string if the object has no get_absolute_url method.
    """
    if tab:
        if hasattr(object, 'get_absolute_url'):
            return object.get_absolute_url(tab)
    else:
        if hasattr(object, 'get_absolute_url'):
            return object.get_absolute_url()
    logger = logging.getLogger('acr.urlhelpers')
    logger.debug("missing absolute_url for %s (%s)" % (object, object.__class__))
    return ''

def edit_url(object, tab=None):
    """
    Helper function to get the edit url from an object.
    Calls get_edit_url method of the object
    Returns empty string if the object has no get_edit_url method.
    """
    if tab:
        if hasattr(object, 'get_edit_url'):
            return object.get_edit_url(tab)
    else:
        if hasattr(object, 'get_edit_url'):
            return object.get_edit_url()
    logger = logging.getLogger('acr.urlhelpers')
    logger.debug("missing edit_url for %s (%s)" % (object, object.__class__))
    return ''

def delete_url(object):
    """
    Helper function to get the delete url from an object.
    Calls get_delete_url method of the object
    Returns empty string if the object has no get_delete_url method.
    """
    if hasattr(object, 'get_delete_url'):
        return object.get_delete_url()
    logger = logging.getLogger('acr.urlhelpers')
    logger.debug("missing delete_url for %s (%s)" % (object, object.__class__))
    return ''
