"""

Versioning utility models

"""
import datetime

from django.core import exceptions
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models, router
from django.db.models import Max
from django.db.models.fields.related import add_lazy_relation
from django.db.models.query import QuerySet, EmptyQuerySet
from django.dispatch import Signal
from django.utils.translation import ugettext_lazy as _
from django import forms
from south.modelsinspector import add_introspection_rules, add_ignored_fields

class CWorkVersion(object):
    """
    Dummy object, representing the work version
    You need to pass this into the version method of the versioning manager if
    you need the current items.
    """
    def __unicode__(cls):
        return u'%s' % _('WorkVersion')

WorkVersion = CWorkVersion()

post_version = Signal(providing_args=['objects'])

class Version(models.Model):
    """
    Version model
    """
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    user = models.ForeignKey(User, related_name='created_versions')
    date = models.DateTimeField()
    published = models.BooleanField(default=False)

    class Meta:
        verbose_name = _('Version')
        verbose_name_plural = _('Versions')
        permissions = (
            ("display_published_versions", "May display published versions"),
            ("display_versions", "May display different versions"),
            ("edit_versions", "May edit versions"),
        )
        ordering = ('-date',)

    def __unicode__(self):
        return u'%s' % self.name

    # urls
    @models.permalink
    def get_absolute_url(self, tab='version'):
        return ('versioning_version', (self.id, tab))

    # Note: we don't define any edit or delete urls

    @models.permalink
    def get_activate_url(self):
        return ('versioning_version_activate', (self.id,))


    # Method overrides

    def save(self, user, propagate=True):
        """
        Custom save, setting date and user info for new created versions
        """
        if not self.pk:
            self.date = datetime.datetime.now()
            self.user = user
            record_exists = False
        else:
            record_exists = True
        super(Version, self).save()
        if propagate and not record_exists:
            post_version.send(sender=self.__class__, instance=self, objects=[])
    save.alters_data = True


    # permissions

    @classmethod
    def may_add(cls, user):
        return user.has_perm('versioning.edit_versions')

    def may_display(self, user):
        """
        for published objects, the user needs the display_published_versions
        permission or the display_versions permission
        for unpublished objects, the user needs the display_versions permission
        """
        if self.published:
            return user.has_perm('versioning.display_published_versions') or user.has_perm('versioning.display_versions')
        return user.has_perm('versioning.display_versions')

    def may_activate(self, user):
        """
        if user may see all versions: ok
        if user may see published versions: ok except for latest published version
        (he already sees that one when no version is activated)
        """
        if user.has_perm('versioning.display_versions'):
            return True
        elif user.has_perm('versioning.display_published_versions'):
            if self.published:
                v = Version.objects.filter(published=True)[0]
                return object != v

    def may_display_collected_items(self, user):
        if self.published:
            return user.has_perm('versioning.display_published_versions') or user.has_perm('versioning.display_versions')
        return user.has_perm('versioning.display_versions')

    @classmethod
    def get_allowed_display_objects(cls, user):
        if user.has_perm('versioning.display_versions'):
            return cls.objects.all()
        elif user.has_perm('versioning.display_published_versions'):
            return cls.objects.filter(published=True)
        return cls.objects.none()

    # Custom methods

    def collect_items(self, user):
        """
        Collect all items that were changed in this version. Check all objects that
        have a foreign key to the Version model for this.
        If a model has a collect_items class method: use it to get the items.
        If not, just get all items that are linked to this version.
        Limit info to what the user is allowed to display
        """
        result = []
        for related in self._meta.get_all_related_objects():
            rel_opts_name = related.get_accessor_name()
            allowed=True
            if hasattr(related.model, 'may_display'):
                allowed = related.model.may_display(user)
            if allowed:
                if hasattr(related.model, 'collect_items'):
                    for item in getattr(related.model, 'collect_items')(self):
                        result.append({
                            'itemtype': item[0],
                            'item': item[1],
                            'change': item[2],
                        })
                else:
                    for item in getattr(self, rel_opts_name).all():
                        change = None
                        if item.new:
                            change = _('New')
                        if item.deleted:
                            change = _('Deleted')
                        result.append({
                            'itemtype': item._meta.verbose_name,
                            'item': item,
                            'change': change
                        })
        return result

    def make_version_for_objects(self, objects):
        """
        objects is a list of lists of objects
        each object will have it's make_version called
        """
        for typelist in objects:
            for item in typelist:
                item.make_version(self)
        post_version.send(sender=self.__class__, instance=self, objects=objects)


    @classmethod
    def exists_changed_objects(cls):
        from django.db.models.loading import get_models
        for cl in get_models():
            if issubclass(cl, VersionedModel):
                c = cl.objects.filter(version__isnull=True, dirty=True).count()
                if c > 0:
                    return True
        return False

class VersionedModelManager(models.Manager):
    """
    Custom manager for versioned items
    """
    def get_query_set(self):
        return VersionedModelQuerySet(self.model)

    def next_v_id(self):
        """
        Get the next available v_id
        """
        max = self.aggregate(max=Max('v_id')).get('max') or 0
        return max + 1

    def dirty(self):
        """
        Filter for dirty items
        """
        return self.get_query_set().filter(dirty=True)

    def get_versionedobject(self, version=None, **kwargs):
        qs = self.get_query_set().filter(**kwargs).get_versionedobjects(version)
        if qs:
            object = qs[0]
        else:
            raise self.model.DoesNotExist
        return object

    def get_versionedobjects(self, version=None):
        return self.get_query_set().get_versionedobjects(version)

    def select_related_versioned(self, *fields):
        return self.get_query_set().select_related_versioned(*fields)

    def none(self):
        return VersionedModelEmptyQuerySet(self.model, using=self._db)

class VersionedModelEmptyQuerySet(EmptyQuerySet):
    def get_versionedobjects(self, version=None):
        return self
    def select_related_versioned(self, *fields):
        return self

class VersionedModelQuerySet(QuerySet):

    def _clone(self, **kwargs):
        # make sure the _version attribute is preserved when cloning
        res = super(VersionedModelQuerySet, self)._clone(**kwargs)
        if hasattr(self, '_version'):
            res._version = self._version
        return res

    def _fill_cache(self, **kwargs):
        super(VersionedModelQuerySet, self)._fill_cache(**kwargs)
        if hasattr(self, '_version'):
            for object in self._result_cache:
                object._version = self._version

    def none(self):
        return self._clone(klass=VersionedModelEmptyQuerySet)

    def get_versionedobjects(self, version=None):
        """
        Retrieve the objects for a certain version.
            This means for each object the version that is older or equal
            to the passed in version.
        Pass in the dummy WorkVersion object to get the unversioned objects.
        None will return an empty queryset.
        """
        if not version:
            return self.none()
        if version == WorkVersion:
            qs = self._clone().exclude(deleted=True).filter(version__isnull=True)
        else:
            tablename = self.model._meta.db_table
            datev = models.DateTimeField().get_db_prep_value(version.date)
            if version.published:
                qs = self._clone().exclude(deleted=True).filter(version__date__isnull=False).extra(
                    where = [
                        """
                        "versioning_version"."date" =
                            (SELECT MAX(C."date")
                             FROM "%s" B LEFT OUTER JOIN "versioning_version" C on B."version_id"=C."id"
                             WHERE B.published and "%s"."v_id" = B."v_id" and C."date" <= %%s)""" %
                        (tablename, tablename)],
                    params=[datev,],
                    )
            else:
                qs = self._clone().exclude(deleted=True).filter(version__date__isnull=False).extra(
                    where = [
                        """
                        "versioning_version"."date" =
                            (SELECT MAX(C."date")
                             FROM "%s" B LEFT OUTER JOIN "versioning_version" C on B."version_id"=C."id"
                             WHERE "%s"."v_id" = B."v_id" and C."date" <= %%s)""" %
                        (tablename, tablename)],
                    params=[datev,],
                    )
        qs._version = version
        return qs

    def select_related_versioned(self, *fields):
        """
        select the related objects for VersionedForeignKey or VersionedGenericForeignKey fields
        inspired by http://www.djangosnippets.org/snippets/1079/

        watch out: this executes the query
        """
        qs = self._clone()

        if not fields:
            fields = [g for g in self.model._meta.fields if isinstance(g, VersionedForeignKey)]
            gfk_fields = [g for g in self.model._meta.virtual_fields if isinstance(g, VersionedGenericForeignKey)]

        else:
            fields = [g for g in self.model._meta.fields if isinstance(g, VersionedForeignKey) and g.name in fields]
            gfk_fields = [g for g in self.model._meta.virtual_fields if isinstance(g, VersionedGenericForeignKey) and g.name in fields]

        version = getattr(self, '_version', None)
        CHUNK_SIZE = 255
        if version:
            # VFK
            for field in fields:
                data_map = {}
                n = field.get_attname()
                v_id_list = [getattr(item,n) for item in qs]
                for i in xrange(0, len(v_id_list), CHUNK_SIZE):
                    v_id_list_chunk = v_id_list[i:i+CHUNK_SIZE]
                    for o in field.to._default_manager.filter(v_id__in=v_id_list_chunk).get_versionedobjects(version=version):
                        data_map[o.v_id] = o
                for item in qs:
                    val = data_map.get(getattr(item, n))
                    if val:
                        setattr(item, field.name, val)

            # VGFK
            ct_map = {}
            item_map = {}
            data_map = {}

            for item in qs:
                for gfk in gfk_fields:
                    ct_id_field = self.model._meta.get_field(gfk.ct_field).column
                    ct_map.setdefault(
                        (getattr(item, ct_id_field)), {}
                        )[getattr(item, gfk.fk_field)] = (gfk.name, item.id)
                item_map[item.id] = item

            for (ct_id), items_ in ct_map.items():
                if (ct_id):
                    ct = ContentType.objects.get_for_id(ct_id)
                    v_id_list = items_.keys()
                    for i in xrange(0, len(v_id_list), CHUNK_SIZE):
                        v_id_list_chunk = v_id_list[i:i+CHUNK_SIZE]
                        for o in ct.model_class().objects.filter(v_id__in=v_id_list_chunk).get_versionedobjects(version=version):
                            (gfk_name, item_id) = items_[o.v_id]
                            data_map[(ct_id, o.v_id)] = o

            for item in qs:
                for gfk in gfk_fields:
                    if (getattr(item, gfk.fk_field) != None):
                        ct_id_field = self.model._meta.get_field(gfk.ct_field).column
    ##                    setattr(item, gfk.name, data_map[(getattr(item, ct_id_field), getattr(item, gfk.fk_field))])
                        val = data_map.get((getattr(item, ct_id_field), getattr(item, gfk.fk_field)))
                        if val:
                            setattr(item, gfk.name, val)
        return qs

class VersionedModel(models.Model):
    """
    Abstract parent model for versionable objects
    """
    version = models.ForeignKey(Version, null=True, blank=True)
    new = models.BooleanField(default=True)
    dirty = models.BooleanField(default=True)
    deleted = models.BooleanField(default=False)
    v_id = models.IntegerField()
    user_created = models.ForeignKey(User, related_name="%(class)s_created")
    user_changed = models.ForeignKey(User, related_name="%(class)s_changed")
    date_created = models.DateTimeField()
    date_changed = models.DateTimeField()
    published = models.BooleanField(default=False)
    published_dirty = models.BooleanField(default=True)

# specific manager
    objects = VersionedModelManager()

    class Meta:
        abstract = True


    # method overrides

    def __init__(self, *args, **kwargs):
        """
        Overridden because of the mismatch between name and attname for VersionedForeignKey fields
        """
        if kwargs:
            fields_iter = iter(self._meta.fields)
            for field in fields_iter:
                if isinstance(field, VersionedForeignKey):
                    val = kwargs.pop(field.name, None)
                    if val:
                        kwargs[field.attname] = val
        super(VersionedModel, self).__init__(*args, **kwargs)

    def save(self, user, set_dirty=True, *args, **kwargs):
        """
        Custom save method
        For totally new items:
            - get a v_id
            - set user and date for creation
        For other items:
            - set dirty flag
            - set user and date for change
        """
        if self.version:
            raise Exception(u'Saving already versioned objects is not allowed')
        else:
            if not self.pk:
                if not self.v_id:
                    # get the next number if not provided
                    self.v_id = self.__class__.objects.next_v_id()
                self.new = True
                self.user_created = user
                self.date_created = datetime.datetime.now()
            if set_dirty:
                self.dirty = True
                self.published_dirty = True
                self.user_changed = user
                self.date_changed = datetime.datetime.now()
            super(VersionedModel, self).save(*args, **kwargs)
            self._version = WorkVersion
    save.alters_data = True

    def delete(self, user, override=False):
        """
        Custom delete method
        Bypasses delete, but sets the deleted flag
        When override=True, falls back to default delete

        """
        if self.version:
            raise Exception(u'Deleting already versioned objects is not allowed')
        else:
            if override or self.new:
                super(VersionedModel, self).delete()
            else:
                self.deleted = True
                self.save(user)
    delete.alters_data = True


    # permissions

    def may_change(self, user):
        """
        Custom change permission method.
        Always returns false for old object states.
        """
        if self.version:
            return False
        return user.has_perm('%s.change_%s' % (self._meta.app_label.lower(), self._meta.object_name.lower()))

    def may_delete(self, user):
        """
        Custom delete permission method.
        Always returns false for old object states.
        """
        if self.version:
            return False
        return user.has_perm('%s.delete_%s' % (self._meta.app_label.lower(), self._meta.object_name.lower()))

    def may_publish(self, user):
        return False

    # Custom methods

    def get_version(self, version, exactversion=False):
        """
        Return the object representing the state of this object at a certain version.
        If exactversion=True, only returns an object if that object was saved in that
        exact version.
        """
        if exactversion:
            res = self.__class__.objects.filter(v_id=self.v_id, version=version)
        else:
            res = self.__class__.objects.filter(v_id=self.v_id).get_versionedobjects(version=version)
        if res:
            return res[0]
        return None

    def is_dirty(self):
        return self.dirty

    def is_published_dirty(self):
        return self.published_dirty

    def history(self):
        """
        Return a queryset of all the old saved states of this object.
        """
        if not hasattr(self, '_history'):
            versionids = list(self.__class__.objects.filter(v_id=self.v_id, version__isnull=False).values_list('version_id', flat=True).distinct())
            if versionids:
                self._history = Version.objects.filter(id__in=versionids).order_by('-date')
            else:
                self._history = Version.objects.none()
        return self._history

    def history_public(self):
        """
        Return a queryset of all the old saved public states of this object.
        """
        if not hasattr(self, '_history'):
            versionids = list(self.__class__.objects.filter(v_id=self.v_id, version__isnull=False, published=True).values_list('version_id', flat=True).distinct())
            if versionids:
                self._history = Version.objects.filter(id__in=versionids).order_by('-date')
            else:
                self._history = Version.objects.none()
        return self._history

    def make_version(self, version):
        """
        Save the objects current state in the version.
        Only dirty objects are saved.
        If the object has foreign keys, save the state of the foreign objects first
        and point the saved state to the correct foreign object states.
        """
        # no action if item is already a versioned item
        toversion = False
        if not self.version:
            if self.dirty:
                toversion=True
            elif version.published and self.published_dirty:
                toversion=True
        if toversion:
            a = self.get_version(version, exactversion=True)
            if not a:
                # reset dirty flag
                self.dirty = False
                if version.published:
                    self.published_dirty = False
                    self.published = True
                # save current pk, so we can reset it later
                currentpk = self.pk
                # setting pk to None will save a new object in the database
                self.pk = None
                self.version=version

                # temporarily save FK values, so we can reset them later
                # is this still relevant ?
                oldvals = {}
                for field in self._meta.fields:
                    if isinstance(field, models.ForeignKey):
                        oldvals[field] = getattr(self, field.name)
                self.prepare_object_for_versioning(version)

    # ? in case we publish, do we need to check that VersionedForeignKey field targets have a published version available ?
    # --> this is not always possible due to the order in which versioning can take place

                # do the real save
                super(VersionedModel, self).save()
                newpk = self.pk

                # reset pk
                self.pk = currentpk
                if self.deleted:
                    # now really delete the item
                    super(VersionedModel,self).delete()
                else:
                    # reset the workversion
                    for key, value in oldvals.items():
                        setattr(self, key.name, value)
                    self.version = None
                    self.new = False
                    self.published = False
                    super(VersionedModel, self).save()
                return self.__class__.objects.get(pk=newpk)
            else:
                # There already is a proper object state, return it
                return a
    make_version.alters_data = True

    def prepare_object_for_versioning(self, version):
        """
        For foreign key fields: make sure the foreign objects are also versioned.
        Point the FK field to the right state.
        """
        for field in self._meta.fields:
            if (field.name not in ['version', 'user_created', 'user_changed', 'content_type'] and isinstance(field, models.ForeignKey)) or isinstance(field, generic.GenericForeignKey):
                relitem = getattr(self, field.name)
                if relitem and isinstance(relitem, VersionedModel):
                    if relitem.dirty:
                        a = relitem.make_version(version)
                    else:
                        a = relitem.get_version(version)
                    setattr(self, field.name, a)


    def copy(self, user, save=True, savekwargs={}, **kwargs):
        import copy
        newitem = copy.copy(self)
        newitem.pk = None
        newitem.v_id = None
        newitem.version = None
        newitem.dirty = True
        newitem.public_dirty = True
        newitem.published = False
        if kwargs:
            for prop in kwargs.keys():
                if hasattr(newitem, prop):
                    setattr(newitem, prop, kwargs.get(prop))
        if save:
            newitem.save(user, **savekwargs)
        return newitem

##    def check_foreignkey_published(self, vfkfields_to_check=[]):
##        """
##        Let's do some validation first
##        Pass in a list of versioned foreign key field names that will NOT be versioned by this method
##        """
##        valid = True
##        errors = []
##        if vfkfields_to_check:
##            for field in self._meta.fields:
##                if field.name in vfkfields_to_check and isinstance(field, VersionedForeignKey):
##                    fobj = getattr(self, field.name)
##                    if fobj:
##                        v_id = fobj.v_id
##                        if field.to.objects.filter(v_id=v_id, published=True).count() == 0:
##                            valid = False
##                            errors.append('Referenced object %s does not have a published version.' % getattr(self, field.name))
##        return valid, errors

    def publish(self, user, instance):
        """
        Create a published version for an object
        """
        instance.make_version_for_objects([[self,],])


# Custom Field definitions

# VersionedForeignKey

class VersionedForeignKeyDescriptor(object):
    """
    Custom descriptor for VFK fields
    """
    def __init__(self, field_with_rel):
        self.field = field_with_rel

    def __get__(self, instance, instance_type=None):
        if instance is None:
            return self

        cache_name = self.field.get_cache_name()
        try:
            return getattr(instance, cache_name)
        except AttributeError:
            val = getattr(instance, self.field.attname)
            if val is None:
                # If NULL is an allowed value, return it.
                if self.field.null:
                    return None
                raise self.field.to.DoesNotExist
            version = getattr(instance, '_version', instance.version) or WorkVersion
            if isinstance(val, self.field.to):
                rel_obj = val
            else:
                try:
                    rel_obj = self.field.to._default_manager.get_versionedobject(v_id=val, version=version)
                except self.field.to.DoesNotExist:
                    return None
            setattr(instance, cache_name, rel_obj)
            return rel_obj

    def __set__(self, instance, value):
        if isinstance(value, self.field.to):
            try:
                val = getattr(value, 'v_id')
            except AttributeError:
                val = None
            setattr(instance, self.field.attname, val)
        else:
            setattr(instance, self.field.attname, value)
        setattr(instance, self.field.get_cache_name(), value)

class VersionedForeignRelatedObjectsDescriptor(object):
    """
    Custom descriptor for the reverse relation of VFK fields
    """
    def __init__(self, model, field):
        self.model = model
        self.field = field

    def __get__(self, instance, instance_type=None):
        if instance is None:
            return self
        return self.create_manager(instance,
                self.model._default_manager.__class__)

    def create_manager(self, instance, superclass):
        rel_field = self.field
        rel_model = self.model

        class RelatedManager(superclass):
            def get_query_set(self):
                db = router.db_for_read(rel_model, instance=instance)
                version = getattr(instance, '_version', instance.version) or WorkVersion
                return superclass.get_query_set(self).using(db).filter(**(self.core_filters)
                        ).get_versionedobjects(version=version)

        manager = RelatedManager()
        manager.core_filters = {self.field.name: getattr(instance, 'v_id')}
        manager.model = self.model

        return manager

class VersionedForeignKey(models.Field):
    empty_strings_allowed = False
    def __init__(self, to, **kwargs):
        self.to = to
        self.related_name = kwargs.pop('related_name', None)
        super(VersionedForeignKey,self).__init__(**kwargs)
        self.db_index = True

    def contribute_to_class(self, cls, name):
        super(VersionedForeignKey, self).contribute_to_class(cls, name)
        setattr(cls, self.name, VersionedForeignKeyDescriptor(self))

        if isinstance(self.to, basestring) or self.to._meta.pk is None:
            def resolve_related_class(field, model, cls):
                field.to = model
                field.do_related_class(model, cls)
            add_lazy_relation(cls, self, self.to, resolve_related_class)
        else:
            self.do_related_class(self.to, cls)

    def do_related_class(self, other, cls):
        if not cls._meta.abstract and self.related_name:
            self.related_name = self.related_name % {
                    'class': cls.__name__.lower(),
                    'app_label': cls._meta.app_label.lower(),
                }
        accessor_name = self.related_name or (cls._meta.object_name.lower() + '_set')
        setattr(other, accessor_name, VersionedForeignRelatedObjectsDescriptor(cls, self))

    def get_attname(self):
        return '%s_id' % self.name

    def get_db_prep_save(self, value, connection):
        if value == '' or value == None:
            return None
        else:
            if isinstance(value, self.to):
                try:
                    val = getattr(value, 'v_id')
                except:
                    val = None
            else:
                val = value
            return self.to._meta.get_field_by_name('v_id')[0].get_db_prep_save(val,
                connection=connection)

    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        def pk_trace(value):
            if isinstance(value, int):
                return value
            return getattr(value, 'v_id', None)
        if lookup_type in ['exact', 'gt', 'lt', 'gte', 'lte']:
            return [pk_trace(value)]
        if lookup_type in ('range', 'in'):
            return [pk_trace(v) for v in value]
        elif lookup_type == 'isnull':
            return []
        raise TypeError("VersionedForeignKey has invalid lookup: %s" % lookup_type)

    def get_internal_type(self):
        return "IntegerField"

    def formfield(self, **kwargs):
        defaults = {
            'form_class': forms.ModelChoiceField,
            'queryset': self.to._default_manager.all(),
            'to_field_name': 'id',
        }
        defaults.update(kwargs)
        return super(VersionedForeignKey, self).formfield(**defaults)

    def validate(self, value, model_instance):
        if value is None:
            return
        super(VersionedForeignKey, self).validate(value, model_instance)
        if not value:
            return
# validation needed here ?

class VersionedManyToManyDescriptor(object):
    """
    Custom descriptor for VersionedManyToManyFields
    """
    def __init__(self, field, reverse=False, normalsource=False, normaltarget=False):
        self.field = field
        self.reverse = reverse
        self.normalsource = normalsource
        self.normaltarget = normaltarget

    def __get__(self, instance, instance_type=None):
        if instance is None:
            return self
        if self.reverse:
            return self.create_manager(instance,
                self.field.model._default_manager.__class__)
        else:
            return self.create_manager(instance,
                self.field.to._default_manager.__class__)

    def create_manager(self, instance, superclass):
        if self.reverse:
            rel_model = self.field.model
            through_model = self.field.through
            source_field_name = self.field.get_target_field_name()
            target_field_name = self.field.get_source_field_name()
            normalsource = self.normaltarget
            normaltarget = self.normalsource
        else:
            rel_model = self.field.to
            through_model = self.field.through
            source_field_name = self.field.get_source_field_name()
            target_field_name = self.field.get_target_field_name()
            normalsource = self.normalsource
            normaltarget = self.normaltarget


        class RelatedManager(superclass):
            def get_query_set(self):
                db = router.db_for_read(through_model, instance=instance)
                if normalsource:
                    version = WorkVersion
                    intermediate = list(through_model._default_manager.get_query_set(
                        ).using(db).filter(**{source_field_name:getattr(instance, 'id')}
                        ).get_versionedobjects(version=version).values_list(target_field_name, flat=True))
                else:
                    version = getattr(instance, '_version', instance.version) or WorkVersion
                    intermediate = list(through_model._default_manager.get_query_set(
                        ).using(db).filter(**{source_field_name:getattr(instance, 'v_id')}
                        ).get_versionedobjects(version=version).values_list(target_field_name, flat=True))
                db = router.db_for_read(rel_model, instance=instance)
                if intermediate:
                    if normaltarget:
                        return superclass.get_query_set(self).using(db).filter(id__in=intermediate
                            ).distinct()
                    else:
                        return superclass.get_query_set(self).using(db).filter(v_id__in=intermediate
                            ).get_versionedobjects(version=version).distinct()
                return superclass.get_query_set(self).none()
        manager = RelatedManager()
        manager.model = rel_model

        return manager

class VersionedManyToManyField(models.Field):
    """
    Subclass for M2M between versioned models
    For now, always use an explicit through model
    Also for now, only for retrieving data.
    """
    description = _("Many-to-many relationship between versioned models")

    def __init__(self, to, **kwargs):
        self.to = to
        self.related_name = kwargs.pop('related_name', None)
        self.source_field_name = kwargs.pop('source_field_name', None)
        self.target_field_name = kwargs.pop('target_field_name', None)
        self.through = kwargs.pop('through', None)
        super(VersionedManyToManyField,self).__init__(**kwargs)

    def contribute_to_class(self, cls, name):
        self.name = name
        self.model = cls
        self.cache_attr = "_%s_cache" % name
        cls._meta.add_virtual_field(self)

        if isinstance(self.to, basestring):
            def resolve_to_class(field, model, cls):
                field.to = model
            add_lazy_relation(cls, self, self.to, resolve_to_class)

        if isinstance(self.through, basestring):
            def resolve_through_class(field, model, cls):
                field.through = model
            add_lazy_relation(cls, self, self.through, resolve_through_class)

        normalsource = not issubclass(cls, VersionedModel)
        normaltarget = not issubclass(self.to, VersionedModel)
        setattr(cls, self.name, VersionedManyToManyDescriptor(self, normalsource=normalsource, normaltarget=normaltarget))
        accessor_name = self.related_name or (cls._meta.object_name.lower() + '_set')
        setattr(self.to, accessor_name, VersionedManyToManyDescriptor(self, reverse=True, normalsource=normalsource, normaltarget=normaltarget))

    def get_source_field_name(self):
        if hasattr(self, 'source_field_name_cache'):
            return getattr(self, 'source_field_name_cache')
        if self.source_field_name:
            setattr(self, 'source_field_name_cache', self.source_field_name)
            return self.source_field_name
        for f in self.through._meta.fields:
            if isinstance(f, VersionedForeignKey):
                if f.to == self.model:
                    setattr(self, 'source_field_name_cache', f.name)
                    return f.name
            if isinstance(f, models.ForeignKey):
                if f.rel.to == self.model:
                    setattr(self, 'source_field_name_cache', f.name)
                    return f.name
        raise Exception, "No source field found on through model %s for model %s" % (self.through, self.model)

    def get_target_field_name(self):
        if hasattr(self, 'target_field_name_cache'):
            return getattr(self, 'target_field_name_cache')
        if self.target_field_name:
            setattr(self, 'target_field_name_cache', self.target_field_name)
            return self.target_field_name
        for f in self.through._meta.fields:
            if isinstance(f, VersionedForeignKey):
                if f.to == self.to:
                    setattr(self, 'target_field_name_cache', f.name)
                    return f.name
            if isinstance(f, models.ForeignKey):
                if f.rel.to == self.to:
                    setattr(self, 'target_field_name_cache', f.name)
                    return f.name
        raise Exception, "No target field found on through model %s for model %s" % (self.through, self.model)

    def db_type(self, connection):
        return None



# VersionedGenericForeignKey

class VersionedGenericForeignKey(generic.GenericForeignKey):
    """
    Special subclass of GFK. Points to the v_id field
    """
    def instance_pre_init(self, signal, sender, args, kwargs, **_kwargs):
        """
        Handles initializing an object with the generic FK instaed of
        content-type/object-id fields.
        """
        if self.name in kwargs:
            value = kwargs.pop(self.name)
            kwargs[self.ct_field] = self.get_content_type(obj=value)
            kwargs[self.fk_field] = getattr(value, 'v_id')

    def __get__(self, instance, instance_type=None):
        if instance is None:
            return self

        try:
            return getattr(instance, self.cache_attr)
        except AttributeError:
            rel_obj = None

            # Make sure to use ContentType.objects.get_for_id() to ensure that
            # lookups are cached (see ticket #5570). This takes more code than
            # the naive ``getattr(instance, self.ct_field)``, but has better
            # performance when dealing with GFKs in loops and such.
            f = self.model._meta.get_field(self.ct_field)
            ct_id = getattr(instance, f.get_attname(), None)
            if ct_id:
                ct = self.get_content_type(id=ct_id, using=instance._state.db)
                version = getattr(instance, '_version', instance.version) or WorkVersion
                try:
                    rel_obj = ct.model_class()._default_manager.get_versionedobject(v_id=getattr(instance, self.fk_field), version=version)
                except exceptions.ObjectDoesNotExist:
                    pass
            setattr(instance, self.cache_attr, rel_obj)
            return rel_obj

    def __set__(self, instance, value):
        if instance is None:
            raise AttributeError(u"%s must be accessed via instance" % self.related.opts.object_name)

        ct = None
        fk = None
        if value is not None:
            ct = self.get_content_type(obj=value)
            fk = getattr(value, 'v_id')

        setattr(instance, self.ct_field, ct)
        setattr(instance, self.fk_field, fk)
        setattr(instance, self.cache_attr, value)



# some rules for SOUTH migrations:

add_introspection_rules([
      (
        (VersionedForeignKey,),
        [],
        {
            "to": ["to", {}],
            "related_name": ["related_name", {"default": None}],
            "db_index": ["db_index", {"default": True}],
        },
      ),
      (
        (VersionedManyToManyField,),
        [],
        {
            "to": ["to", {}],
            "related_name": ["related_name", {"default": None}],
            "through": ["through", {}],
            "db_table": ["db_table", {"default": None}],
        },
      ),
    ],
    ["^utils\.versioning\.models"]
)
add_ignored_fields(["^utils\.versioning\.models\.VersionedGenericForeignKey"])
